package top.lifuquan.config;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;
import top.lifuquan.service.CronService;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.Executors;

@Slf4j
@Configuration
public class ScheduledConfig implements SchedulingConfigurer {
    // 由配置文件指定线程数
    @Value("${scheduled.pool.size:2}")
    private Integer poolSize;
    @Value("${scheduled.cronVolte:0 30 11 * * ?}")
    private String cronVolte;
    @Value("${scheduled.cron5g:0 30 13 * * ?}")
    private String cron5g;
    @Value("${scheduled.delay:1}")
    private long delay;

    private final SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy年MM月dd日");

    @Resource
    private CronService cronService;

    @Override
    public void configureTasks(ScheduledTaskRegistrar scheduledTaskRegistrar) {
        scheduledTaskRegistrar.setScheduler(Executors.newScheduledThreadPool(poolSize));
        scheduledTaskRegistrar.addTriggerTask(
                () -> {
                    // 回溯延迟时间
                    Date date = new Date(new Date().getTime() - delay * 1000 * 3600 * 24);
                    log.info("开始执行{}Volte终端入库...", dateFormat.format(date));
                    cronService.runVolteTask(date);
                    log.info("{}Volte终端入库执行结束...", dateFormat.format(date));
                },
                triggerContext -> new CronTrigger(cronVolte).nextExecutionTime(triggerContext)
        );
        scheduledTaskRegistrar.addTriggerTask(
                () -> {
                    // 回溯延迟时间
                    Date date = new Date(new Date().getTime() - delay * 1000 * 3600 * 24);
                    log.info("开始执行{}5G终端入库...", dateFormat.format(date));
                    cronService.run5gTask(date);
                    log.info("{}5G终端入库执行结束...", dateFormat.format(date));
                },
                triggerContext -> new CronTrigger(cron5g).nextExecutionTime(triggerContext)
        );
    }
}
