package top.lifuquan.util;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;
import org.apache.commons.csv.CSVRecord;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.zip.GZIPInputStream;

@Slf4j
public class CsvUtil {

    private static Reader reader = null;
    private static CSVPrinter printer = null;

    /**
     * 对csv文件进行去重
     *
     * @param csvIn                输入文件路径
     * @param csvOut               输出文件路径
     * @param distinctColumnNumber 去重依据的列的编号，从0开始
     * @param delimiter            csv文件的分割符
     */
    public static boolean distinct(String csvIn, String csvOut, int distinctColumnNumber, String delimiter) {
        Iterable<CSVRecord> records = null;
        try {
            reader = getReader(csvIn);
            // todo: 这里需要检查reader是否为null 或者检查csvIn是否存在
            records = CSVFormat.DEFAULT.builder()
                    .setDelimiter(delimiter)
                    .build().parse(reader);
            log.debug("第一次遍历文件，筛选第 " + distinctColumnNumber + " 列不重复的项，重复值将保留最后一条记录....");
            // 将文件加入map，对每个不同的 distinctColumnNumber ，仅会保留最后1条
            Map<String, Integer> map = new HashMap<>();
            int count = 0;
            for (CSVRecord record : records) {
                // map中记录的是distinctColumnNumber最后一行的行号
                map.put(record.get(distinctColumnNumber), count);
                count++;
            }
            log.debug("筛选完成，共筛选 " + count + " 记录,其中不重复的记录有 " + map.size() + " 条");
            // 需要从头开始重新遍历文件
            reader = getReader(csvIn);
            records = read(csvIn, delimiter);
            // 输出
            printer = getPrinter(csvOut, delimiter);
            log.debug("准备第二次遍历输入，将不重复记录写入 " + csvOut);
            count = 0;
            for (CSVRecord record : records) {
                // 如果map中记录的行号是当前的行号，就可以写入输出文件
                if (map.get(record.get(distinctColumnNumber)) == count) {
                    printer.printRecord(record);
                }
                count++;
            }
            printer.flush();
            log.debug("已完成去重");
            return true;
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            close();
        }
        return false;
    }

    private static Iterable<CSVRecord> read(String csvIn, String delimiter) throws IOException {
        reader = getReader(csvIn);
        return CSVFormat.DEFAULT.builder()
                .setDelimiter(delimiter)
                .build().parse(reader);
    }


    private static CSVPrinter getPrinter(String csvOut, String delimiter) throws IOException {
        Appendable out = new PrintWriter(csvOut);
        return CSVFormat.DEFAULT.builder()
                .setDelimiter(delimiter)
                .build().print(out);
    }

    private static void close() {
        if (reader != null) {
            try {
                reader.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        if (printer != null) {
            try {
                printer.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 需要处理的文件可能是文本或者gz压缩格式
     *
     * @param csvIn 输入文件路径
     * @return Reader
     * @throws IOException
     */
    private static Reader getReader(String csvIn) throws IOException {
        if (csvIn.toLowerCase().endsWith(".gz")) {
            // 这里识别一下gz压缩文件
            return new InputStreamReader(new GZIPInputStream(new FileInputStream(csvIn)));
        } else {
            return new FileReader(csvIn);
        }
    }
}
