package top.lifuquan.util;

import lombok.extern.slf4j.Slf4j;
import top.lifuquan.pojo.TaskRecord;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class ProcessUtil {

    private static final String scpShell = "scp_file.sh";
    private static final String gpLoad = "gpload.sh";

    private static Process process = null;
    private static BufferedReader reader = null;

    public static TaskRecord gpLoad(String date, TaskRecord record) {
        if (!new File(gpLoad).exists()) {
            // scp的sh文件丢失
            log.info("用于scp的sh文件丢失。。。。");
        }
        try {
            log.debug("准备调用gpload");
            process = Runtime.getRuntime().exec("sh " + gpLoad + " " + date + " " + record.getTaskType());
            reader = getReader();
            List<String> lines = new ArrayList<>();
            String line;
            while ((line = reader.readLine()) != null) {
                log.debug(line);
                lines.add(line);
            }
            return getTaskRecordFromResult(lines, record);
        } catch (IOException e) {
            e.printStackTrace();
            record.setTaskStatus("GPLOAD失败");
            return record;
        } finally {
            destroy();
        }
    }

    private static TaskRecord getTaskRecordFromResult(List<String> lines, TaskRecord record) {
        for (int i = 0; i < lines.size(); i++) {
            if (lines.get(i).contains("|INFO|rows Inserted")) {
                // 找到第一行 对应的是插入数据的数量
                // s4可以判断gpload是否成功
                String s4 = lines.get(i + 3);
                if (s4.endsWith("succeeded")) {
                    // 插入数量
                    String s1 = lines.get(i);
                    s1 = s1.substring(s1.indexOf('=') + 2);
                    // 更新数量
                    String s2 = lines.get(i + 1);
                    s2 = s2.substring(s2.indexOf('=') + 2);
                    // 错误数量
                    String s3 = lines.get(i + 2);
                    s3 = s3.substring(s3.indexOf('=') + 2);
                    record.setInserted(Integer.parseInt(s1));
                    record.setUpdated(Integer.parseInt(s2));
                    record.setError(Integer.parseInt(s3));
                    record.setTaskStatus("任务完成");
                } else {
                    record.setTaskStatus("GPLOAD失败");
                }
            }
        }
        return record;
    }

    public static boolean scpFile(String fileName, String localPath) {
        if (!new File(scpShell).exists()) {
            // scp的sh文件丢失
            log.info("用于scp的sh文件丢失。。。。");
            return false;
        }
        // 拼接本地文件路径
        String localFilePath = Paths.get(localPath, fileName).toString();
        try {
            // 执行本地sh文件进行下载
            process = Runtime.getRuntime().exec("sh " + scpShell + " " + fileName + " " + localPath);
            reader = getReader();
            // 识别完成度的百分比
            Pattern pattern = Pattern.compile("\\d+%");
            String line;
            Matcher matcher;
            while ((line = reader.readLine()) != null) {
                if (line.contains("No such file or directory")) {
                    // 在源端文件不存在
                    log.debug("在源端不存在文件" + fileName);
                }
                matcher = pattern.matcher(line);
                if (matcher.find()) {
                    if (matcher.group().equals("100%")) {
                        // 只有下载到100%才算完成
                        log.info("文件SCP完成,已下载至 " + localFilePath);
                        return true;
                    }
                }
            }
            // 文件未下载成功，删除文件
            log.debug("文件未下载完成，可能的原因是在源端不存在文件或下载超时，将删除未完全下载的文件 " + localFilePath);
            rmFile(localFilePath);
        } catch (IOException e) {
            // 遇到异常也可能会遗留未完全下载的文件
            e.printStackTrace();
            rmFile(localFilePath);
        } finally {
            destroy();
        }
        return false;
    }

    private static BufferedReader getReader() {
        return new BufferedReader(new InputStreamReader(process.getInputStream()));
    }

    private static void rmFile(String localFilePath) {
        File file = new File(localFilePath);
        if (file.exists()) {
            file.delete();
        }
    }

    private static void destroy() {
        if (reader != null) {
            try {
                reader.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        if (process != null) {
            process.destroy();
        }
    }
}
